// Copyright 2006, 2007 The Apache Software Foundation
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

package org.apache.tapestry.internal.bindings;

import org.apache.tapestry.Binding;
import org.apache.tapestry.ComponentResources;
import org.apache.tapestry.PropertyConduit;
import org.apache.tapestry.ioc.Location;
import org.apache.tapestry.ioc.internal.util.TapestryException;
import org.apache.tapestry.services.BindingFactory;
import org.apache.tapestry.services.PropertyConduitSource;

/**
 * Binding factory for reading and updating JavaBean properties.
 * <p/>
 * Expression are evaluated via a {@link PropertyConduit}, which is generated by {@link PropertyConduitSource} (which
 * therefore defines the expression language).
 */
public class PropBindingFactory implements BindingFactory
{
    private final PropertyConduitSource _source;

    public PropBindingFactory(PropertyConduitSource propertyConduitSource)
    {
        _source = propertyConduitSource;
    }

    public Binding newBinding(String description, ComponentResources container,
                              ComponentResources component, String expression, Location location)
    {
        Object target = container.getComponent();
        Class targetClass = target.getClass();

        try
        {
            PropertyConduit conduit = _source.create(targetClass, expression);

            String toString = String.format("PropBinding[%s %s(%s)]", description, container
                    .getCompleteId(), expression);

            return new PropBinding(target, conduit, toString, location);
        }
        catch (Throwable ex)
        {
            throw new TapestryException(ex.getMessage(), location, ex);
        }
    }
}
